# -*- coding: utf-8 -*-

__all__ = ["kepler", "quad_solution_vector", "contact_points"]


import numpy as np

from .. import driver


def kepler(mean_anomaly, eccentricity):
    mean_anomaly = np.ascontiguousarray(mean_anomaly, dtype=np.float64)
    eccentricity = np.ascontiguousarray(eccentricity, dtype=np.float64)
    sinf = np.empty_like(mean_anomaly)
    cosf = np.empty_like(mean_anomaly)
    driver.solve_kepler(mean_anomaly, eccentricity, sinf, cosf)
    return sinf, cosf


def quad_solution_vector(b, r):
    b = np.ascontiguousarray(b, dtype=np.float64)
    r = np.ascontiguousarray(r, dtype=np.float64)
    s = np.empty(r.shape + (3,), dtype=np.float64)
    driver.quad_solution_vector(b, r, s)
    return s


def contact_points(a, e, cosw, sinw, cosi, sini, L):
    a = np.ascontiguousarray(a, dtype=np.float64)
    e = np.ascontiguousarray(e, dtype=np.float64)
    cosw = np.ascontiguousarray(cosw, dtype=np.float64)
    sinw = np.ascontiguousarray(sinw, dtype=np.float64)
    cosi = np.ascontiguousarray(cosi, dtype=np.float64)
    sini = np.ascontiguousarray(sini, dtype=np.float64)
    L = np.ascontiguousarray(L, dtype=np.float64)
    M_left = np.empty_like(a)
    M_right = np.empty_like(a)
    flag = np.empty_like(a, dtype=np.int32)
    driver.contact_points(
        a, e, cosw, sinw, cosi, sini, L, M_left, M_right, flag
    )
    return M_left, M_right, flag



# -*- coding: utf-8 -*-

__all__ = ["asarray"]

import aesara_theano_fallback.tensor as tt
from aesara_theano_fallback import aesara


def asarray(a, dtype=None, order=None):
    if dtype is None:
        dtype = aesara.config.floatX
    return tt.as_tensor_variable(a).astype(dtype)



# -*- coding: utf-8 -*-

__all__ = ["quad_limbdark_light_curve", "kepler"]

import numpy as np

from .numpy import ops


def get_cl(u1, u2):
    c = np.array([1 - u1 - 1.5 * u2, u1 + 2 * u2, -0.25 * u2])
    c /= np.pi * (c[0] + c[1] / 1.5)
    return c


def custom_broadcast_arrays(*arrays):
    b = np.broadcast(*arrays)
    return tuple(
        np.ascontiguousarray(np.broadcast_to(a, b.shape), dtype=np.float64)
        for a in arrays
    )


def quad_limbdark_light_curve(u1, u2, b, r):
    b, r = custom_broadcast_arrays(b, r)
    c = get_cl(float(u1), float(u2))
    return np.dot(ops.quad_solution_vector(b, r), c) - 1


def kepler(mean_anomaly, eccentricity):
    mean_anomaly, eccentricity = custom_broadcast_arrays(
        mean_anomaly, eccentricity
    )
    return ops.kepler(mean_anomaly, eccentricity)



# -*- coding: utf-8 -*-

__all__ = ["get_mean_and_true_anomaly", "check_contact_points"]

import numpy as np

from .numpy.ops import kepler


def get_mean_and_true_anomaly(eccentricity, eccentric_anomaly):
    M = eccentric_anomaly - eccentricity * np.sin(eccentric_anomaly)
    f = 2 * np.arctan(
        np.sqrt((1 + eccentricity) / (1 - eccentricity))
        * np.tan(0.5 * eccentric_anomaly)
    )
    return M, f


def _get_b2(sinf, cosf, a, e, cosw, sinw, cosi, sini):
    e2 = e ** 2
    factor = (a * (e2 - 1) / (e * cosf + 1)) ** 2
    return factor * (
        cosi ** 2 * (cosw * sinf + sinw * cosf) ** 2
        + (cosw * cosf - sinw * sinf) ** 2
    )


def check_contact_points(func, L, a, b, e, w):
    target = L ** 2
    cosw = np.cos(w)
    sinw = np.sin(w)

    incl_factor = (1 + e * sinw) / (1 - e ** 2)
    cosi = incl_factor * b * L / a
    if np.abs(cosi) >= 1:
        return
    i = np.arccos(cosi)
    sini = np.sin(i)

    M1, M2, flag = func(a, e, cosw, sinw, cosi, sini, L)
    if np.any(flag):
        return

    sinf1, cosf1 = kepler(M1, e)
    sinf2, cosf2 = kepler(M2, e)

    fs = [(sinf1, cosf1), (sinf2, cosf2)]
    assert np.all(np.isfinite(fs))
    for sinf, cosf in fs:
        np.testing.assert_allclose(
            target,
            _get_b2(sinf, cosf, a, e, cosw, sinw, cosi, sini),
            atol=1e-5,
        )
